function F = bdf1_dae_step_fo(X,xk,tkp1,h,daetype,parameter)
# Die Loesung X eines BDF2-Schrittes
# ist eine Nullstelle von bdf2_dae_step_fo.
#
# input:  X Startnaeherung fuer die Stufen
#         x_k Loesung des vorherigen BDF-Schrittes,
#         t_k vorheriger Zeitpunkt, h Schrittweite,
#         daetype ('proper' or 'classic'),
#         parameter fuer Parameter von f und d
# output: F Funktionswert der Nullstellenfunktion
#
# benutzte Programme: Nutzerfunktionen f 
# der Form f = f(y,x,t,daetype,parameter)
#
# Aufruf: F = bdf1_dae_step_fo(X,xk,tkp1,h,daetype,parameter)

dX = bdf2_dae_do_GearPetzold(X,tkp1,daetype,parameter);
dxk = bdf2_dae_do_GearPetzold(xk,tkp1-h,daetype,parameter);

ykp1 = (dX - dxk)/h;

F = bdf1_dae_fo_GearPetzold(ykp1,X,tkp1,daetype,parameter);
endfunction

function d = bdf2_dae_do_GearPetzold(x,t,daetype,parameter)
    eta = parameter;
    if strcmp(daetype,'proper')
        d = x(1) + eta*t*x(2);
    else # daetype = 'classic'
        d = x;
    end
end

function f = bdf1_dae_fo_GearPetzold(y,x,t,daetype,parameter)
    eta = parameter;
    if strcmp(daetype,'proper')
        f1 = y + x(2);
        f2 = x(1) + eta*t*x(2) - exp(-t);
    else # daetype = 'classic'
        f1 = y(1) + eta*t*y(2) + (1+eta)*x(2);
        f2 = x(1) + eta*t*x(2) - exp(-t);
    end
    f = [f1 f2];
end

function [x,t] = bdf1_dae(t0,tf,x0,h,daetype,parameter)
# Programm 9.1 (bdf1_dae.m)
# zur Loesung von DAEs f((d(x,t))',x,t)=0
# BDF2-Verfahren
# input:  Intervallgrenzen t0,tf,
#         Anfangswert x0=x(t0), Schrittweite h
#         daetype ('proper' or 'classic'),
#         parameter fuer Parameter von f und d
# Beispiel: t0 = 0; tf = 3; x0 = [1.0,1.0]; h = 0.05; daetype = 'classic'; parameter = 0.1;
# output: Loesung x(t) an den Punkten t
# benutztes Programm: bdf2_dae_step_fo
# Aufruf: [x,t] = bdf1_dae(t0,tf,x0,h,daetype,parameter)

n = floor((tf - t0)/h);
[r,m] = size(x0);

t = zeros(n+1,1); x = zeros(n+1,m);
t(1) = t0; 
x(1,:) = x0; 
xkm1 = x0;
xk = x0;
tkp1 = t0 + h;;

options = optimset('Display','off');
for k = 1:n
	# berechne Nullstelle von bdf2_step_f
	# und nehme x_k als Startnaeherung:
	Xstart = xk;
	fun = @(x)bdf1_dae_step_fo(x,xk,tkp1,h,daetype,parameter);
	xkp1 = fsolve(fun,Xstart,options);
	t(k+1) = tkp1; # speichere Zeitpunkt t_{k+1}
	x(k+1,:) = xkp1; # speichere Loesung x_{k+1}
	tkp1 = tkp1 + h; # update t_{k+1} fuer naechsten Schritt
	xk = xkp1; # update x_{k} fuer naechsten Schritt
endfor
endfunction

# Hauptprogramm zum Aufruf von bdf1_dae
t0 = 0; tf = 4.0; h = 0.5e-01; x0 = [1.0,1.0]; parameter = -0.1;
daetype1 = 'proper';
# daetype2 = 'classic';
[x,t] = bdf1_dae(t0,tf,x0,h,daetype1,parameter);
# [y,t] = bdf1_dae(t0,tf,x0,h,daetype2,parameter);

figure(1)
plot(t,x(:,1))
xlabel('t')
ylabel('x1')
legend('proper')
title('Gear-Petzold-DAE')
figure(2)
plot(t,x(:,2))
xlabel('t')
ylabel('x2')
title('Gear-Petzold-DAE')
legend('proper')

